""" 方法调用相关参数定义 仅为测试使用 真实设备时请根据真实情况实现 """
import numpy as np

from core.joylink import JoylinkSDK, DeviceCallbackHandler
from protocol.payload.functions import FuncCall
from protocol.payload.properties import PropGet, PropSet
from protocol.payload.thingmodel import ThingModel, ThingModelPostRes
from util.log import getLogger

logger = getLogger("example.device")

DEV_FUN_TYPE_NONE: int = 0  # 无入参
DEV_FUN_TYPE_INOUT: int = 1  # 直接返回入参


class DevFunCall:
    key: str
    type: int
    outNum: int
    out: dict
    outKey: list[str]


# 定义设备结构体
class Device:
    sdk: JoylinkSDK  # sdk 句柄
    deviceId: str  # 设备标识
    thingModel: ThingModel  # 物模型
    version: np.uint16  # 属性版本
    propNum: int  # 属性个数
    prop: dict  # 设备属性
    funNum: int  # 方法个数
    func: list[DevFunCall]  # 方法调用

    def __init__(self, deviceId: str, sdk: JoylinkSDK):
        """ 设备初始化 """
        self.sdk = sdk
        self.deviceId = deviceId

        # 物模型
        self.thingModel = ThingModel()
        self.thingModel.id = "urn:user-spec-v1:thing-model:EC-control-test:bbd1cfc0279091635311806bcbc348f1"
        self.thingModel.version = "v1.0.0"

        """
         ***************************************************************************
         * 通用 属性
         ***************************************************************************/
         """
        self.version = 0
        self.propNum = 2
        self.prop = {
            "selfmode.propA": False,
            "selfmode.propB": 15
        }

        """
         ***************************************************************************
         * 通用 方法调用
         ***************************************************************************
        """
        self.funNum = 1
        func0 = DevFunCall()
        func0.key = "selfmode.funcA"
        func0.type = DEV_FUN_TYPE_INOUT
        func0.outNum = 1
        func0.out = {
            "out1": 20,
        }
        func0.outKey = ["out1"]
        self.func: list[DevFunCall] = [func0]


# 全局变量：设备
deviceEntity: Device


def DeviceInitialise(sdk: JoylinkSDK, deviceId: str = None):
    if deviceId is None:
        logger.error("deviceId is undefined. cannot initialise.")
        return
    global deviceEntity
    deviceEntity = Device(deviceId, sdk)


def GetDevice() -> Device:
    return deviceEntity


class MyDeviceCallbackHandler(DeviceCallbackHandler):
    """ 自定义继承回调函数控制类，实现具体回调业务处理逻辑 """

    def onConnect(self):
        """ 设备回调 SDK连接成功回调函数 """
        logger.info("Enter Device CallBackOnConnect")
        from example.device.entity import entityOnlineStatusPost, entityThingModelPost, entityPropertiesPost, \
            entityEventPost
        # 设备上线
        entityOnlineStatusPost(True)
        entityThingModelPost()

        # 通用属性上报
        entityPropertiesPost()

        # 通用事件上报
        entityEventPost()

        logger.info("Leave Device CallBackOnConnect")

    def onThMdPostRes(self, objName: str, serviceKey: str, inRes: ThingModelPostRes):
        """ 设备回调 物模型上报响应 """
        logger.info("Enter Device CallBackOnThingModelPostRes")
        logger.info(f"\tDeviceId:\t{inRes.deviceId}")
        logger.info(f"\tMessageId:\t{inRes.messageId}")
        logger.info(f"\tTimestamp:\t{inRes.timestamp}")
        logger.info(f"\tCode:\t{inRes.code}")
        logger.info(f"\tMessage:\t{inRes.message}")
        logger.info("Leave Device CallBackOnThingModelPostRes")

    def onPropSet(self, objName: str, serviceKey: str, inSet: PropSet):
        """ 设备回调 属性设置 """
        logger.info("Enter Device CallBackOnPropertiesSet")
        if serviceKey == "":
            # 通用属性设置
            from example.device.entity import entityPropertiesSet
            entityPropertiesSet(inSet)
        elif serviceKey == "connection_agent":
            # connection agent 属性设置
            logger.info("connection agent properties set unsupported yet!!")
        else:
            pass
        logger.info("Leave Device CallBackOnPropertiesSet")

    def onPropGet(self, objName: str, serviceKey: str, inGet: PropGet):
        """ 设备回调 属性获取 """
        logger.info("Enter Device CallBackOnPropertiesGet")
        if serviceKey == "":
            # 通用属性设置
            from example.device.entity import entityPropertiesGet
            entityPropertiesGet(inGet)
        elif serviceKey == "connection_agent":
            # connection agent 属性设置
            logger.info("connection agent properties set unsupported yet!!")
        else:
            pass
        logger.info("Leave Device CallBackOnPropertiesGet")

    def onFuncCall(self, objName: str, serviceKey: str, inCall: FuncCall):
        """ 设备回调 方法调用 """
        logger.info("Enter Device CallBackOnFunctionsCall")
        if serviceKey == "":
            # 通用方法调用
            from example.device.entity import entityFunctionsCall
            entityFunctionsCall(inCall)
        elif serviceKey == "connection_agent":
            # connection agent 属性设置
            logger.info("connection agent properties set unsupported yet!!")
        else:
            pass
        logger.info("Leave Device CallBackOnFunctionsCall")
