from example.device.device import GetDevice
from protocol.payload.events import OnlineStatus, EventPost, EventPostEvent
from protocol.payload.functions import FuncCall, FuncCallRes, FuncCallResFuncs
from protocol.payload.properties import PropPost, PropGetRes, PropGet, PropSet, PropSetRes
from protocol.payload.thingmodel import ThingModelPost
from util.log import getLogger

logger = getLogger("example.entity")


# 实体模型 上下线状态上报
def entityOnlineStatusPost(isOnline: bool):
    """ 实体模型 上下线状态上报 """
    logger.info("Enter Device entityOnlineStatusPost")
    post = OnlineStatus()
    deviceEntity = GetDevice()
    post.deviceId = deviceEntity.deviceId
    if isOnline:
        post.events = "connected"
    else:
        post.events = "disconnected"

    post.thingModel = deviceEntity.thingModel

    # 上报
    deviceEntity.sdk.OnlineStatusPost("device", "", post)
    logger.info("Leave Device entityOnlineStatusPost")


# 实体模型 物模型上报
def entityThingModelPost():
    """ 实体模型 物模型上报 """
    logger.info("Enter Device entityThingModelPost")
    post = ThingModelPost()
    deviceEntity = GetDevice()
    post.deviceId = deviceEntity.deviceId
    post.model = deviceEntity.thingModel

    # 上报
    deviceEntity.sdk.ThingModelPost("device", "", post)
    logger.info("Leave Device entityThingModelPost")


# 通用属性 上报
def entityPropertiesPost():
    """ 通用属性 上报 """
    logger.info("Enter Device entityPropertiesPost")
    post = PropPost()
    deviceEntity = GetDevice()
    post.deviceId = deviceEntity.deviceId
    post.version = deviceEntity.version
    post.properties = {}
    for k, v in deviceEntity.prop.items():
        post.properties[k] = v

    # 上报
    deviceEntity.sdk.PropertiesPost("device", "", post)
    logger.info("Leave Device entityPropertiesPost")


# 通用属性设置
def entityPropertiesSet(inSet: PropSet):
    """ 通用属性设置 """
    logger.info("Enter Device entityPropertiesSet")

    # response
    outRes = PropSetRes()
    outRes.deviceId = inSet.deviceId
    outRes.messageId = inSet.messageId
    if inSet.version == 65535:
        outRes.version = 0
    else:
        outRes.version = inSet.version + 1

    outRes.code = 200
    outRes.properties = {}
    deviceEntity = GetDevice()
    for k, v in inSet.properties.items():
        deviceEntity.prop[k] = v
        outRes.properties[k] = deviceEntity.prop[k]

    deviceEntity.sdk.PropertiesSetResponse("device", "", outRes)
    logger.info("Leave Device entityPropertiesSet")


# 通用属性获取
def entityPropertiesGet(inGet: PropGet):
    """ 通用属性获取 """
    logger.info("Enter Device entityPropertiesGet")

    # response
    outRes = PropGetRes()
    outRes.deviceId = inGet.deviceId
    outRes.messageId = inGet.messageId
    deviceEntity = GetDevice()
    outRes.version = deviceEntity.version
    outRes.code = 200

    lenProp = len(inGet.properties)
    if lenProp > 0:
        outRes.properties = {}
        for ii in range(0, lenProp, 1):
            outRes.properties[inGet.properties[ii]] = deviceEntity.prop[inGet.properties[ii]]
    else:
        # 获取所有
        outRes.properties = {}
        for k, v in deviceEntity.prop.items():
            outRes.properties[k] = v

    deviceEntity.sdk.PropertiesGetResponse("device", "", outRes)
    logger.info("Leave Device entityPropertiesGet")


# 通用属性获取
def entityFunctionsCall(inCall: FuncCall):
    """ 通用属性获取 """
    logger.info("Enter Device entityFunctionsCall")

    # response
    outRes = FuncCallRes()
    outRes.deviceId = inCall.deviceId
    outRes.messageId = inCall.messageId

    outRes.functions = []
    for item in inCall.functions:
        funcAdd = FuncCallResFuncs()
        funcAdd.key = item.key
        funcAdd.out = item.inParam
        outRes.functions.append(item)

    deviceEntity = GetDevice()
    deviceEntity.sdk.FunctionsCallResponse("device", "", outRes)
    logger.info("Leave Device entityFunctionsCall")


# 通用事件 上报
def entityEventPost():
    """ 通用事件 上报 """
    logger.info("Enter Device entityEventPost")
    post = EventPost()
    deviceEntity = GetDevice()
    post.deviceId = deviceEntity.deviceId
    post.events = [EventPostEvent()]
    post.events[0].key = "event_test"
    post.events[0].parameters = {
        "param": "this is a test event"
    }

    # 上报
    deviceEntity.sdk.EventsPost("device", "", post)
    logger.info("Leave Device entityEventPost")
