from enum import Enum

import numpy as np

# 消息主题前缀与版本定义
from util.JsonWrapper import JsonObj

prefix: str = "$iot/v1/"
maxParts: int = 8
minParts: int = 5


class TopicType(Enum):
    # 消息类型
    TYPE_NAME_PROP: str = "properties"
    TYPE_NAME_EVT: str = "events"
    TYPE_NAME_FUNC: str = "functions"
    TYPE_NAME_REG: str = "register"
    TYPE_NAME_AUTH: str = "auth"
    TYPE_NAME_HB: str = "heartbeat"
    TYPE_NAME_THMD: str = "thing-model"

    # 消息主题列表枚举
    AUTH_POST: np.int32 = 0
    AUTH_POST_RES: np.int32 = 1

    THINGMODEL_POST: np.int32 = 2
    THINGMODEL_POST_RES: np.int32 = 3

    EVT_ONLINE: np.int32 = 4
    EVT_OFFLINE: np.int32 = 5

    HB_POST: np.int32 = 6
    HB_POST_RES: np.int32 = 7
    HB_SET: np.int32 = 8

    EVT_POST: np.int32 = 9

    PROP_SET: np.int32 = 10
    PROP_SET_RES: np.int32 = 11
    PROP_GET: np.int32 = 12
    PROP_GET_RES: np.int32 = 13
    PROP_POST: np.int32 = 14

    FUNC_CALL: np.int32 = 15
    FUNC_CALL_RES: np.int32 = 16

    REG: np.int32 = 17
    REG_RES: np.int32 = 18


# 消息主题结构体定义
class Topic(JsonObj):
    retain: str = None
    version: str = None
    objName: str = None
    deviceId: str = None
    serviceKey: str = None
    typeName: str = None
    command: str = None
    response: str = None


# 获取消息主题字符串 return str
def TopicGet(tType: np.int32, objName: str, deviceId: str, servicKey: str) -> str:
    matchDict: dict = {
        TopicType.AUTH_POST: "/auth/post",
        TopicType.AUTH_POST_RES: "/auth/post/response",
        TopicType.EVT_ONLINE: "/events/online",
        TopicType.EVT_OFFLINE: "/events/offline",
        TopicType.HB_POST: "/hearbeat/post",
        TopicType.HB_POST_RES: "/hearbeat/post/response",
        TopicType.HB_SET: "/hearbeat/set",
        TopicType.EVT_POST: "/events/post",
        TopicType.PROP_SET: "/properties/set",
        TopicType.PROP_SET_RES: "/properties/set/response",
        TopicType.PROP_GET: "/properties/get",
        TopicType.PROP_GET_RES: "/properties/get/response",
        TopicType.PROP_POST: "/properties/post",
        TopicType.FUNC_CALL: "/functions/call",
        TopicType.FUNC_CALL_RES: "/functions/call/response",
        TopicType.REG: "/register",
        TopicType.REG_RES: "/register/response",
        TopicType.THINGMODEL_POST: "/thing-model/post",
        TopicType.THINGMODEL_POST_RES: "/thing-model/post/response"
    }
    tailStr: str = matchDict[tType]
    if tailStr is None:
        tailStr = ""
    retTopic: str
    if len(servicKey) > 0:
        retTopic = prefix + objName + "/" + deviceId + "/" + servicKey + tailStr
    else:
        retTopic = prefix + objName + "/" + deviceId + tailStr
    return retTopic


# 消息主题解析
def TopicParse(inTopic: str) -> Topic:
    isService: bool = True
    ii: int
    retTopic = Topic()
    strArr = inTopic.split("/")
    lenArr = len(strArr)
    if minParts <= lenArr <= maxParts:
        # retain
        retTopic.retain = strArr[0]

        # version
        retTopic.version = strArr[1]

        # object namespace
        retTopic.objName = strArr[2]

        # deviceId
        retTopic.deviceId = strArr[3]

        # 判断是否含有service key
        if strArr[4] == TopicType.TYPE_NAME_PROP:
            isService = False
        elif strArr[4] == TopicType.TYPE_NAME_EVT:
            isService = False
        elif strArr[4] == TopicType.TYPE_NAME_FUNC:
            isService = False
        elif strArr[4] == TopicType.TYPE_NAME_REG:
            isService = False
        elif strArr[4] == TopicType.TYPE_NAME_AUTH:
            isService = False
        elif strArr[4] == TopicType.TYPE_NAME_HB:
            isService = False
        elif strArr[4] == TopicType.TYPE_NAME_THMD:
            isService = False
        else:
            pass

        ii = 4
        if isService:
            retTopic.serviceKey = strArr[ii]
            ii += 1

        # type name
        retTopic.typeName = strArr[ii]
        ii += 1

        # command
        if lenArr > ii:
            retTopic.command = strArr[ii]
            ii += 1
            # response
            if lenArr > ii:
                retTopic.response = strArr[ii]

    return retTopic
