#
# Copyright (c) 2019 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import gzip
import json
import string
import threading
import time
import random
from typing import Type
from paho.mqtt import client as mqtt_client

import numpy as np

from util.JsonWrapper import MyJsonDecoder, MyJsonEncoder, JsonObj

lock = threading.RLock()


# 获取messageId
def GetMessageId(prefix: str) -> str:
    """ 生成消息ID """
    lock.acquire()
    ret: str
    try:
        rand = random.randrange(100000, 999999)
        time_now = np.int64(time.time())
        m_second = str(time.time_ns())[-9:]
        ret = prefix + str(rand) + "-" + str(time_now) + "-" + str(f64(int(m_second), 9))
        # ret = prefix + "-" + str(time_now) + "-" + str(f64(m_second, 9))
    finally:
        lock.release()
    return ret


def GetTimestamp() -> np.int64:
    """ 获取时间戳：秒 """
    return np.int64(time.time())


def f64(n, x):
    """输入任意十进制数n要转换的进制x， 返回对应转换后的数。注意进制范围仅限2到64, 并且进制的值必须小于待转换的十进制数"""
    # 1、判断是否超出范围，是则返回ERROR
    if x < 2 or x > 64 or x > n:  # 进制超出范围或者进制x大于要转换的十进制数
        return 'ERROR'

    # 2、拼接出64进制下，1-64的数值表示符号
    li1 = [i for i in range(10)]  # 拼接出0到9的列表
    li2 = list(string.ascii_letters)  # 拼接出所有小写字母和大写字母的列表
    li3 = ['@', '_']
    li = li1 + li2 + li3
    # li = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
    # 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K',
    # 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '@', '_']

    # 3、进制转换
    result = ''
    while True:
        m = n // x  # 商
        r = n % x  # 余数
        # print(r)
        if n < x:
            result = str(li[n]) + result  # 跳出循环前，把最后的商，拼接在最前
            break
        result = str(li[r]) + result  # 把每次的余数拼接到前面
        n = m

    return result


def parseJson(s, *, clazz: Type = None, cls=MyJsonDecoder, object_hook=None, parse_float=None,
              parse_int=None, parse_constant=None, object_pairs_hook=None, **kw):
    """ 转换json字符串为对象 """
    jsonDict: dict = json.loads(s, cls=cls, object_hook=object_hook, parse_float=parse_float, parse_int=parse_int,
                                parse_constant=parse_constant,
                                object_pairs_hook=object_pairs_hook, **kw)
    if clazz is not None:
        e = clazz()
        if issubclass(clazz, JsonObj):
            trans = e.get_trans()
            for k, v in trans.items():
                if v in jsonDict:
                    jsonDict[k] = jsonDict[v]
                    jsonDict.pop(v, None)
        else:
            pass
        # set attr
        for k, v in jsonDict.items():
            e.__setattr__(k, v)
        return e
    else:
        return jsonDict


def jsonToStr(obj, *, skipkeys=False, ensure_ascii=True, check_circular=True,
              allow_nan=True, cls=MyJsonEncoder, indent=None, separators=None,
              default=None, sort_keys=False, **kw):
    """ 转换对象为json字符串 """
    return json.dumps(obj, skipkeys=skipkeys, ensure_ascii=ensure_ascii, check_circular=check_circular,
                      allow_nan=allow_nan, cls=cls, indent=indent, separators=separators,
                      default=default, sort_keys=sort_keys, **kw)


def GetClientId(client: mqtt_client.Client) -> str:
    """ 获取 mqtt clientId """
    cid: bytes = client.__getattribute__("_client_id")
    return cid.decode("UTF-8")


def DoGZip(content: str, iszip: bool):
    """ 使用gzip压缩 """
    if iszip:
        return gzip.compress(str.encode(content, "UTF-8"))
    else:
        return content
